#include <SPI.h>
#include <WiFi101.h>
#include <WiFiUdp.h>
#include <OSCMessage.h>

int status = WL_IDLE_STATUS;
#include "arduino_secrets.h" 
char ssid[] = SECRET_SSID;
char pass[] = SECRET_PASS;
int keyIndex = 0;
unsigned int localPort = 5500;
unsigned int remoteOSC1 = 5600;
unsigned int remoteOSC2 = 5700;
char packetBuffer[63];
WiFiUDP Udp;
int pwm[2][7];
int main_timer = 0;
char HEX_char1, HEX_char2;

void setup() {
  WiFi.setPins(8,7,4,2);
  Serial.begin(115200);
//  while (!Serial) { }
  if (WiFi.status() == WL_NO_SHIELD) while (true);
  pwm[0][0]=5; pwm[0][1]=6; pwm[0][2]=9; pwm[0][3]=10;
  pwm[0][4]=11; pwm[0][5]=12; pwm[0][6]=13; 
  for(int i=0;i<7;i++){
    pinMode(pwm[0][i], OUTPUT);
    pwm[1][i] = 127;
    analogWrite(pwm[0][i], pwm[1][i]);
  }
  while (status != WL_CONNECTED) {
    status = WiFi.begin(ssid, pass);
    delay(5000);
  }
  IPAddress ip = WiFi.localIP();
  if(Serial){
    Serial.print("\nSSID: ");
    Serial.println(WiFi.SSID());
    Serial.print("IP Address: ");
    Serial.println(ip);
    Serial.println("OSC Start !");
  }
  Udp.begin(localPort);
  for(int i=0;i<7;i++){
    pwm[1][i] = 0;
    analogWrite(pwm[0][i], pwm[1][i]);
  }
}

void loop() {
  int packetSize = Udp.parsePacket();
  if (packetSize){
    int len = Udp.read(packetBuffer, 63);
    if (len > 0) packetBuffer[len] = 0;
    Udp.beginPacket(Udp.remoteIP(), remoteOSC2);
    OSCMessage msg2(packetBuffer);
    for(int i=0;i<7;i++){
      HEX_char1 = packetBuffer[i*2];
      HEX_char2 = packetBuffer[i*2+1];
      pwm[1][i] = 16*conv_HEX(HEX_char1) + conv_HEX(HEX_char2);
      if(pwm[1][i] == 1700){
//        WiFi.setSleepMode(M2M_PS_H_AUTOMATIC, 1);
        WiFi.lowPowerMode();
//        WiFi.setSleepMode(M2M_PS_MANUAL, 1);
//        WiFi.requestSleep(100000);
        for(int i=0;i<7;i++){
          pwm[1][i] = 0;
          analogWrite(pwm[0][i], pwm[1][i]);
        }
        while(1){}
      }
      analogWrite(pwm[0][i], pwm[1][i]);
      msg2.add(pwm[1][i]);
    }
    msg2.send(Udp);
    Udp.endPacket();
    msg2.empty();
  }
  if (++main_timer > 24331){
    main_timer = 0;
    Udp.beginPacket(Udp.remoteIP(), remoteOSC1);
    OSCMessage msg1("/analog");
    msg1.add((int32_t)analogRead(0));
    msg1.add((int32_t)analogRead(1));
    msg1.add((int32_t)analogRead(2));
    msg1.add((int32_t)analogRead(3));
    msg1.add((int32_t)analogRead(4));
    msg1.add((int32_t)analogRead(5));
    msg1.send(Udp);
    Udp.endPacket();
    msg1.empty();
  }
}

int conv_HEX(char input_HEX){
  switch(input_HEX){
    case('0'):
      return(0); 
    case('1'):
      return(1); 
    case('2'):
      return(2); 
    case('3'):
      return(3); 
    case('4'):
      return(4); 
    case('5'):
      return(5); 
    case('6'):
      return(6); 
    case('7'):
      return(7); 
    case('8'):
      return(8); 
    case('9'):
      return(9); 
    case('A'):
      return(10); 
    case('B'):
      return(11); 
    case('C'):
      return(12); 
    case('D'):
      return(13); 
    case('E'):
      return(14); 
    case('F'):
      return(15);
    case('s'):
      return(100);
    case('l'):
      return(100);
    return(0);
  }
}
